@@ -23,7 +23,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
-#include <sys/poll.h>
+#include <poll.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -39,6 +39,8 @@
 #include "log.h"
 #include "sock.h"
 #include "ud_socket.h"
+#include "libc_compat.h"
+
 #include "event.h"
 /*
  * What is a rule?  It's polymorphic, pretty much.
@@ -63,8 +65,10 @@ struct rule_list {
 	struct rule *head;
 	struct rule *tail;
 };
-static struct rule_list cmd_list;
+
+static struct rule_list drop_list;
 static struct rule_list client_list;
+static struct rule_list cmd_list;
 
 /* rule routines */
 static void enlist_rule(struct rule_list *list, struct rule *r);
@@ -84,6 +88,8 @@ static int safe_write(int fd, const char *buf, int len);
 static char *parse_cmd(const char *cmd, const char *event);
 static int check_escapes(const char *str);
 
+extern const char *dropaction;
+
 /*
  * read in all the configuration files
  */
@@ -174,7 +180,11 @@ acpid_read_conf(const char *confdir)
         /* fd is closed by parse_file() */
 		r = parse_file(fd_rule, file);
 		if (r) {
-			enlist_rule(&cmd_list, r);
+			/* if this is a drop rule */
+			if (!strcmp(r->action.cmd, dropaction))
+				enlist_rule(&drop_list, r);
+			else
+				enlist_rule(&cmd_list, r);
 			nrules++;
 		}
 		free(file);
@@ -225,6 +235,15 @@ acpid_cleanup_rules(int do_detach)
 		p = next;
 	}
 
+	/* drop the drop rules */
+	p = drop_list.head;
+	while (p) {
+		next = p->next;
+		delist_rule(&drop_list, p);
+		free_rule(p);
+		p = next;
+	}
+
 	unlock_rules();
 
 	return 0;
@@ -529,17 +548,17 @@ acpid_close_dead_clients(void)
 }
 
 /*
- * the main hook for propogating events
+ * the main hook for propagating events
  */
 int
 acpid_handle_event(const char *event)
 {
 	struct rule *p;
 	int nrules = 0;
-	struct rule_list *ar[] = { &client_list, &cmd_list, NULL };
+	struct rule_list *ar[] = { &drop_list, &client_list, &cmd_list, NULL };
 	struct rule_list **lp;
 
-	/* make an event be atomic wrt known signals */
+	/* make an event atomic wrt known signals */
 	lock_rules();
 
 	/* scan each rule list for any rules that care about this event */
@@ -558,7 +577,16 @@ acpid_handle_event(const char *event)
 				}
 				nrules++;
 				if (p->type == RULE_CMD) {
-					do_cmd_rule(p, event);
+					if (do_cmd_rule(p, event) == DROP_EVENT) {
+						/* Abort processing if event matches drop rule */
+						if (logevents)
+							acpid_log(LOG_INFO, "event dropped");
+						/* Skip the remaining rules. */
+						while (*++lp)
+							;
+						--lp;
+						break;
+					}
 				} else if (p->type == RULE_CLIENT) {
 					do_client_rule(p, event);
 				} else {
@@ -631,6 +659,9 @@ do_cmd_rule(struct rule *rule, const char *event)
 	int status;
 	const char *action;
 
+	if (!strcmp(rule->action.cmd, dropaction))
+		return DROP_EVENT;
+
 	pid = fork();
 	switch (pid) {
 	case -1:
